@@ -88,6 +88,7 @@ struct cfg_postparser {
 	int (*func)();
 };
 
+extern struct list postparsers;
 extern int cfg_maxpconn;
 extern int cfg_maxconn;
 extern char *cfg_scope;
@@ -126,6 +127,7 @@ void free_email_alert(struct proxy *p);
 const char *cfg_find_best_match(const char *word, const struct list *list, int section, const char **extra);
 int warnifnotcap(struct proxy *proxy, int cap, const char *file, int line, const char *arg, const char *hint);
 int failifnotcap(struct proxy *proxy, int cap, const char *file, int line, const char *arg, const char *hint);
+void cfg_dump_registered_keywords();
 
 /* simplified way to define a section parser */
 #define REGISTER_CONFIG_SECTION(name, parse, post)                            \
