@@ -137,8 +137,8 @@ struct qcs {
 /* QUIC application layer operations */
 struct qcc_app_ops {
 	int (*init)(struct qcc *qcc);
-	int (*attach)(struct qcs *qcs);
-	int (*decode_qcs)(struct qcs *qcs, int fin, void *ctx);
+	int (*attach)(struct qcs *qcs, void *conn_ctx);
+	int (*decode_qcs)(struct qcs *qcs, int fin);
 	size_t (*snd_buf)(struct stconn *sc, struct buffer *buf, size_t count, int flags);
 	void (*detach)(struct qcs *qcs);
 	int (*finalize)(void *ctx);
