@@ -24,11 +24,15 @@
 #define _HAPROXY_STATS_H
 
 #include <haproxy/api.h>
-#include <haproxy/applet-t.h>
+#include <haproxy/listener-t.h>
 #include <haproxy/stats-t.h>
-#include <haproxy/stream_interface-t.h>
 #include <haproxy/tools-t.h>
 
+struct channel;
+struct buffer;
+struct proxy;
+struct appctx;
+struct htx;
 
 /* These two structs contains all field names and descriptions according to
  * the the number of entries in "enum stat_field" and "enum info_field"
@@ -55,7 +59,6 @@ int stats_fill_sv_stats(struct proxy *px, struct server *sv, int flags,
 int stats_fill_be_stats(struct proxy *px, int flags, struct field *stats, int len,
 			enum stat_field *selected_field);
 
-void stats_io_handler(struct stream_interface *si);
 int stats_emit_raw_data_field(struct buffer *out, const struct field *f);
 int stats_emit_typed_data_field(struct buffer *out, const struct field *f);
 int stats_emit_field_tags(struct buffer *out, const struct field *f,
