@@ -746,53 +746,6 @@ static inline const char *ist_find_ctl(const struct ist ist)
 	return NULL;
 }
 
-/* Returns a pointer to the first character found <ist> that belongs to the
- * range [min:max] inclusive, or NULL if none is present. The function is
- * optimized for strings having no such chars by processing up to sizeof(long)
- * bytes at once on architectures supporting efficient unaligned accesses.
- * Despite this it is not very fast (~0.43 byte/cycle) and should mostly be
- * used on low match probability when it can save a call to a much slower
- * function. Will not work for characters 0x80 and above. It's optimized for
- * min and max to be known at build time.
- */
-static inline const char *ist_find_range(const struct ist ist, unsigned char min, unsigned char max)
-{
-	const union { unsigned long v; } __attribute__((packed)) *u;
-	const char *curr = (void *)ist.ptr - sizeof(long);
-	const char *last = curr + ist.len;
-	unsigned long l1, l2;
-
-	/* easier with an exclusive boundary */
-	max++;
-
-	do {
-		curr += sizeof(long);
-		if (curr > last)
-			break;
-		u = (void *)curr;
-		/* add 0x<min><min><min><min>..<min> then subtract
-		 * 0x<max><max><max><max>..<max> to the value to generate a
-		 * carry in the lower byte if the byte contains a lower value.
-		 * If we generate a bit 7 that was not there, it means the byte
-		 * was min..max.
-		 */
-		l2  = u->v;
-		l1  = ~l2 & ((~0UL / 255) * 0x80); /* 0x808080...80 */
-		l2 += (~0UL / 255) * min;          /* 0x<min><min>..<min> */
-		l2 -= (~0UL / 255) * max;          /* 0x<max><max>..<max> */
-	} while ((l1 & l2) == 0);
-
-	last += sizeof(long);
-	if (__builtin_expect(curr < last, 0)) {
-		do {
-			if ((unsigned char)(*curr - min) < (unsigned char)(max - min))
-				return curr;
-			curr++;
-		} while (curr < last);
-	}
-	return NULL;
-}
-
 /* looks for first occurrence of character <chr> in string <ist> and returns
  * the tail of the string starting with this character, or (ist.end,0) if not
  * found.
@@ -910,7 +863,7 @@ static inline int istissame(const struct ist ist1, const struct ist ist2)
 static inline struct ist istalloc(const size_t size)
 {
 	/* Note: do not use ist2 here, as it triggers a gcc11 warning.
-	 * ‘<unknown>’ may be used uninitialized [-Werror=maybe-uninitialized]
+	 * <unknown> may be used uninitialized [-Werror=maybe-uninitialized]
 	 *
 	 * This warning is reported because the uninitialized memory block
 	 * allocated by malloc should not be passed to a const argument as in
@@ -938,12 +891,15 @@ static inline void istfree(struct ist *ist)
  */
 static inline struct ist istdup(const struct ist src)
 {
-	/* Allocate 1 extra byte to add an extra \0 delimiter. */
-	struct ist dst = istalloc(src.len + 1);
+	const size_t src_size = src.len;
+
+	/* Allocate at least 1 byte to allow duplicating an empty string with
+	 * malloc implementations that return NULL for a 0-size allocation.
+	 */
+	struct ist dst = istalloc(src_size ? src_size : 1);
 
 	if (isttest(dst)) {
-		istcpy(&dst, src, src.len);
-		dst.ptr[dst.len] = '\0';
+		istcpy(&dst, src, src_size);
 	}
 
 	return dst;
