@@ -133,6 +133,8 @@ DECLARE_STATIC_POOL(pool_head_h3c, "h3c", sizeof(struct h3c));
 #define H3_SF_UNI_NO_H3 0x00000002  /* unidirectional stream does not carry H3 frames */
 
 struct h3s {
+	struct h3c *h3c;
+
 	enum h3s_t type;
 	int demux_frame_len;
 	int demux_frame_type;
@@ -334,6 +336,8 @@ static int h3_is_frame_valid(struct h3c *h3c, struct qcs *qcs, uint64_t ftype)
 static int h3_headers_to_htx(struct qcs *qcs, struct ncbuf *buf, uint64_t len,
                              char fin)
 {
+	struct h3s *h3s = qcs->ctx;
+	struct h3c *h3c = h3s->h3c;
 	struct buffer htx_buf = BUF_NULL;
 	struct buffer *tmp = get_trash_chunk();
 	struct htx *htx = NULL;
@@ -343,14 +347,17 @@ static int h3_headers_to_htx(struct qcs *qcs, struct ncbuf *buf, uint64_t len,
 	struct ist meth = IST_NULL, path = IST_NULL;
 	//struct ist scheme = IST_NULL, authority = IST_NULL;
 	struct ist authority = IST_NULL;
-	int hdr_idx;
+	int hdr_idx, ret;
 
 	TRACE_ENTER(H3_EV_RX_FRAME|H3_EV_RX_HDR, qcs->qcc->conn, qcs);
 
 	/* TODO support buffer wrapping */
 	BUG_ON(ncb_head(buf) + len >= ncb_wrap(buf));
-	if (qpack_decode_fs((const unsigned char *)ncb_head(buf), len, tmp,
-	                    list, sizeof(list) / sizeof(list[0])) < 0) {
+	ret = qpack_decode_fs((const unsigned char *)ncb_head(buf), len, tmp,
+	                    list, sizeof(list) / sizeof(list[0]));
+	if (ret < 0) {
+		TRACE_ERROR("QPACK decoding error", H3_EV_RX_FRAME|H3_EV_RX_HDR, qcs->qcc->conn, qcs);
+		h3c->err = -ret;
 		return -1;
 	}
 
@@ -383,8 +390,10 @@ static int h3_headers_to_htx(struct qcs *qcs, struct ncbuf *buf, uint64_t len,
 	flags |= HTX_SL_F_XFER_LEN;
 
 	sl = htx_add_stline(htx, HTX_BLK_REQ_SL, flags, meth, path, ist("HTTP/3.0"));
-	if (!sl)
+	if (!sl) {
+		h3c->err = H3_INTERNAL_ERROR;
 		return -1;
+	}
 
 	if (fin)
 		sl->flags |= HTX_SL_F_BODYLESS;
@@ -412,8 +421,10 @@ static int h3_headers_to_htx(struct qcs *qcs, struct ncbuf *buf, uint64_t len,
 	if (fin)
 		htx->flags |= HTX_FL_EOM;
 
-	if (!qc_attach_sc(qcs, &htx_buf))
+	if (!qc_attach_sc(qcs, &htx_buf)) {
+		h3c->err = H3_INTERNAL_ERROR;
 		return -1;
+	}
 
 	/* buffer is transferred to the stream connector and set to NULL
 	 * except on stream creation error.
@@ -576,11 +587,11 @@ static size_t h3_parse_settings_frm(struct h3c *h3c, const struct ncbuf *rxbuf,
  *
  * Returns 0 on success else non-zero.
  */
-static int h3_decode_qcs(struct qcs *qcs, int fin, void *ctx)
+static int h3_decode_qcs(struct qcs *qcs, int fin)
 {
 	struct ncbuf *rxbuf = &qcs->rx.ncbuf;
-	struct h3c *h3c = ctx;
 	struct h3s *h3s = qcs->ctx;
+	struct h3c *h3c = h3s->h3c;
 	ssize_t ret;
 
 	h3_debug_printf(stderr, "%s: STREAM ID: %lu\n", __func__, qcs->id);
@@ -656,8 +667,14 @@ static int h3_decode_qcs(struct qcs *qcs, int fin, void *ctx)
 			break;
 		case H3_FT_HEADERS:
 			ret = h3_headers_to_htx(qcs, rxbuf, flen, last_stream_frame);
-			/* TODO handle error reporting. Stream closure required. */
-			if (ret < 0) { ABORT_NOW(); }
+			if (ret < 0) {
+				/* TODO for some error, it may be preferable to
+				 * only close the stream once RESET_STREAM is
+				 * supported.
+				 */
+				qcc_emit_cc_app(qcs->qcc, h3c->err);
+				return -1;
+			}
 			break;
 		case H3_FT_CANCEL_PUSH:
 		case H3_FT_PUSH_PROMISE:
@@ -1024,7 +1041,7 @@ size_t h3_snd_buf(struct stconn *sc, struct buffer *buf, size_t count, int flags
 	return total;
 }
 
-static int h3_attach(struct qcs *qcs)
+static int h3_attach(struct qcs *qcs, void *conn_ctx)
 {
 	struct h3s *h3s;
 
@@ -1035,6 +1052,8 @@ static int h3_attach(struct qcs *qcs)
 		return 1;
 
 	qcs->ctx = h3s;
+	h3s->h3c = conn_ctx;
+
 	h3s->demux_frame_len = 0;
 	h3s->demux_frame_type = 0;
 	h3s->flags = 0;
@@ -1145,7 +1164,7 @@ static void h3_trace(enum trace_level level, uint64_t mask,
 		chunk_appendf(&trace_buf, " : qcc=%p(F)", qcc);
 
 		if (qcs)
-			chunk_appendf(&trace_buf, " qcs=%p(%lu)", qcs, qcs->id);
+			chunk_appendf(&trace_buf, " qcs=%p(%llu)", qcs, (ull)qcs->id);
 	}
 }
 
