@@ -139,15 +139,14 @@ static int __http_find_header(const struct htx *htx, const void *pattern, struct
 		if (flags & HTTP_FIND_FL_FULL)
 			goto next_blk;
 		v = htx_get_blk_value(htx, blk);
-		p = ctx->value.ptr + ctx->value.len + ctx->lws_after;
+		p = istend(ctx->value) + ctx->lws_after;
 		v.len -= (p - v.ptr);
 		v.ptr  = p;
 		if (!v.len)
 			goto next_blk;
 		/* Skip comma */
 		if (*(v.ptr) == ',') {
-			v.ptr++;
-			v.len--;
+			v = istnext(v);
 		}
 
 		goto return_hdr;
@@ -196,7 +195,8 @@ static int __http_find_header(const struct htx *htx, const void *pattern, struct
 				if (istlen(n) < istlen(name))
 					goto next_blk;
 
-				n = ist2(istptr(n) + istlen(n) - istlen(name), istlen(name));
+				n = ist2(istend(n) - istlen(name),
+					 istlen(name));
 				if (!isteqi(n, name))
 					goto next_blk;
 				break;
@@ -216,13 +216,13 @@ static int __http_find_header(const struct htx *htx, const void *pattern, struct
 		ctx->lws_before = 0;
 		ctx->lws_after = 0;
 		while (v.len && HTTP_IS_LWS(*v.ptr)) {
-			v.ptr++;
-			v.len--;
+			v = istnext(v);
 			ctx->lws_before++;
 		}
 		if (!(flags & HTTP_FIND_FL_FULL))
-			v.len = http_find_hdr_value_end(v.ptr, v.ptr + v.len) - v.ptr;
-		while (v.len && HTTP_IS_LWS(*(v.ptr + v.len - 1))) {
+			v.len = http_find_hdr_value_end(v.ptr, istend(v)) - v.ptr;
+
+		while (v.len && HTTP_IS_LWS(*(istend(v) - 1))) {
 			v.len--;
 			ctx->lws_after++;
 		}
@@ -388,9 +388,6 @@ int http_replace_req_uri(struct htx *htx, const struct ist uri)
 		goto fail;
 
 	sl = http_get_stline(htx);
-	ALREADY_CHECKED(sl); /* the stline exists because http_replace_stline() succeded */
-	sl->flags &= ~HTX_SL_F_NORMALIZED_URI;
-
 	if (!http_update_host(htx, sl, uri))
 		goto fail;
 
@@ -410,12 +407,14 @@ int http_replace_req_path(struct htx *htx, const struct ist path, int with_qs)
 	struct htx_sl *sl = http_get_stline(htx);
 	struct ist meth, uri, vsn, p;
 	size_t plen = 0;
+	struct http_uri_parser parser;
 
 	if (!sl)
 		return 0;
 
 	uri = htx_sl_req_uri(sl);
-	p = http_get_path(uri);
+	parser = http_uri_parser_init(uri);
+	p = http_parse_path(&parser);
 	if (!isttest(p))
 		p = uri;
 	if (with_qs)
@@ -433,7 +432,7 @@ int http_replace_req_path(struct htx *htx, const struct ist path, int with_qs)
 	vsn = ist2(temp->area + meth.len, HTX_SL_REQ_VLEN(sl));
 
 	chunk_memcat(temp, uri.ptr, p.ptr - uri.ptr);         /* uri: host part */
-	chunk_memcat(temp, path.ptr, path.len);               /* uri: new path */
+	chunk_istcat(temp, path);                             /* uri: new path */
 	chunk_memcat(temp, p.ptr + plen, p.len - plen);       /* uri: QS part */
 	uri = ist2(temp->area + meth.len + vsn.len, uri.len - plen + path.len);
 
@@ -458,16 +457,14 @@ int http_replace_req_query(struct htx *htx, const struct ist query)
 	uri = htx_sl_req_uri(sl);
 	q = uri;
 	while (q.len > 0 && *(q.ptr) != '?') {
-		q.ptr++;
-		q.len--;
+		q = istnext(q);
 	}
 
 	/* skip the question mark or indicate that we must insert it
 	 * (but only if the format string is not empty then).
 	 */
 	if (q.len) {
-		q.ptr++;
-		q.len--;
+		q = istnext(q);
 	}
 	else if (query.len > 1)
 		offset = 0;
@@ -577,8 +574,7 @@ int http_replace_header_value(struct htx *htx, struct http_hdr_ctx *ctx, const s
 	}
 
 	ctx->blk = blk;
-	ctx->value.ptr = v.ptr + off;
-	ctx->value.len = data.len;
+	ctx->value = ist2(v.ptr + off, data.len);
 	ctx->lws_before = ctx->lws_after = 0;
 
 	return 1;
@@ -677,8 +673,7 @@ int http_remove_header(struct htx *htx, struct http_hdr_ctx *ctx)
 	htx_change_blk_value_len(htx, blk, v.len-len);
 
 	/* Finally update the ctx */
-	ctx->value.ptr = start;
-	ctx->value.len = 0;
+	ctx->value = ist2(start, 0);
 	ctx->lws_before = ctx->lws_after = 0;
 
 	return 1;
@@ -694,9 +689,11 @@ int http_update_authority(struct htx *htx, struct htx_sl *sl, const struct ist h
 {
 	struct buffer *temp = get_trash_chunk();
 	struct ist meth, vsn, uri, authority;
+	struct http_uri_parser parser;
 
 	uri = htx_sl_req_uri(sl);
-	authority = http_get_authority(uri, 1);
+	parser = http_uri_parser_init(uri);
+	authority = http_parse_authority(&parser, 1);
 	if (!authority.len)
 		return 0;
 
@@ -712,8 +709,8 @@ int http_update_authority(struct htx *htx, struct htx_sl *sl, const struct ist h
 	vsn = ist2(temp->area + meth.len, HTX_SL_REQ_VLEN(sl));
 
 	chunk_memcat(temp, uri.ptr, authority.ptr - uri.ptr);
-	chunk_memcat(temp, host.ptr, host.len);
-	chunk_memcat(temp, authority.ptr + authority.len, uri.ptr + uri.len - (authority.ptr + authority.len));
+	chunk_istcat(temp, host);
+	chunk_memcat(temp, istend(authority), istend(uri) - istend(authority));
 	uri = ist2(temp->area + meth.len + vsn.len, host.len + uri.len - authority.len); /* uri */
 
 	return http_replace_stline(htx, meth, uri, vsn);
@@ -731,9 +728,11 @@ int http_update_host(struct htx *htx, struct htx_sl *sl, const struct ist uri)
 {
 	struct ist authority;
 	struct http_hdr_ctx ctx;
+	struct http_uri_parser parser = http_uri_parser_init(uri);
 
-	if (!uri.len || uri.ptr[0] == '/' ||  uri.ptr[0] == '*') {
-		// origin-form or a asterisk-form (RFC7320 #5.3.1 and #5.3.4)
+	if (parser.format == URI_PARSER_FORMAT_EMPTY ||
+	    parser.format == URI_PARSER_FORMAT_ASTERISK ||
+	    parser.format == URI_PARSER_FORMAT_ABSPATH) {
 		sl->flags &= ~HTX_SL_F_HAS_AUTHORITY;
 	}
 	else {
@@ -744,7 +743,7 @@ int http_update_host(struct htx *htx, struct htx_sl *sl, const struct ist uri)
 			if (uri.len > 4 && (uri.ptr[0] | 0x20) == 'h')
 				sl->flags |= ((uri.ptr[4] == ':') ? HTX_SL_F_SCHM_HTTP : HTX_SL_F_SCHM_HTTPS);
 
-			authority = http_get_authority(uri, 1);
+			authority = http_parse_authority(&parser, 1);
 			if (!authority.len)
 				goto fail;
 		}
@@ -918,10 +917,10 @@ int http_str_to_htx(struct buffer *buf, struct ist raw, char **errmsg)
 
 	h1m_init_res(&h1m);
 	h1m.flags |= H1_MF_NO_PHDR;
-	ret = h1_headers_to_hdr_list(raw.ptr, raw.ptr + raw.len,
+	ret = h1_headers_to_hdr_list(raw.ptr, istend(raw),
 				     hdrs, sizeof(hdrs)/sizeof(hdrs[0]), &h1m, &h1sl);
 	if (ret <= 0) {
-		memprintf(errmsg, "unable to parse headers (error offset: %d)", h1m.err_pos);
+		memprintf(errmsg, "unabled to parse headers (error offset: %d)", h1m.err_pos);
 		goto error;
 	}
 
@@ -1056,7 +1055,7 @@ static int http_htx_init(void)
 			continue;
 		}
 
-		raw = ist2(http_err_msgs[rc], strlen(http_err_msgs[rc]));
+		raw = ist(http_err_msgs[rc]);
 		if (!http_str_to_htx(&chk, raw, &errmsg)) {
 			ha_alert("Internal error: invalid default message for HTTP return code %d: %s.\n",
 				 http_err_codes[rc], errmsg);
@@ -1113,6 +1112,9 @@ static void http_htx_deinit(void)
 		LIST_DELETE(&http_rep->list);
 		release_http_reply(http_rep);
 	}
+
+	for (rc = 0; rc < HTTP_ERR_SIZE; rc++)
+		chunk_destroy(&http_err_chunks[rc]);
 }
 
 REGISTER_CONFIG_POSTPARSER("http_htx", http_htx_init);
@@ -1568,7 +1570,6 @@ struct http_reply *http_parse_http_reply(const char **args, int *orig_arg, struc
 			fd = -1;
 			obj[objlen] = '\0';
 			reply->type = HTTP_REPLY_LOGFMT;
-			LIST_INIT(&reply->body.fmt);
 			cur_arg++;
 		}
 		else if (strcmp(args[cur_arg], "lf-string") == 0) {
@@ -1585,7 +1586,6 @@ struct http_reply *http_parse_http_reply(const char **args, int *orig_arg, struc
 			obj = strdup(args[cur_arg]);
 			objlen = strlen(args[cur_arg]);
 			reply->type = HTTP_REPLY_LOGFMT;
-			LIST_INIT(&reply->body.fmt);
 			cur_arg++;
 		}
 		else if (strcmp(args[cur_arg], "hdr") == 0) {
@@ -1740,6 +1740,7 @@ int http_scheme_based_normalize(struct htx *htx)
 	struct http_hdr_ctx ctx;
 	struct htx_sl *sl;
 	struct ist uri, scheme, authority, host, port;
+	struct http_uri_parser parser;
 
 	sl = http_get_stline(htx);
 
@@ -1748,13 +1749,14 @@ int http_scheme_based_normalize(struct htx *htx)
 
 	uri = htx_sl_req_uri(sl);
 
-	scheme = http_get_scheme(uri);
+	parser = http_uri_parser_init(uri);
+	scheme = http_parse_scheme(&parser);
 	/* if no scheme found, no normalization to proceed */
 	if (!isttest(scheme))
 		return 0;
 
 	/* Extract the port if present in authority */
-	authority = http_get_authority(uri, 1);
+	authority = http_parse_authority(&parser, 1);
 	port = http_get_host_port(authority);
 	if (!isttest(port)) {
 		/* if no port found, no normalization to proceed */
@@ -2095,7 +2097,7 @@ static int proxy_check_errors(struct proxy *px)
 
 			/* unknown http-errors section */
 			if (&http_errs->list == &http_errors_list) {
-				ha_alert("config : proxy '%s': unknown http-errors section '%s' (at %s:%d).\n",
+				ha_alert("proxy '%s': unknown http-errors section '%s' (at %s:%d).\n",
 					 px->id, conf_err->info.errorfiles.name, conf_err->file, conf_err->line);
 				err |= ERR_ALERT | ERR_FATAL;
 				free(conf_err->info.errorfiles.name);
