@@ -139,7 +139,7 @@ struct qcs *qcs_new(struct qcc *qcc, uint64_t id, enum qcs_type type)
 
 	qcs->id = qcs->by_id.key = id;
 	if (qcc->app_ops->attach) {
-		if (qcc->app_ops->attach(qcs))
+		if (qcc->app_ops->attach(qcs, qcc->ctx))
 			goto err;
 	}
 
@@ -432,9 +432,19 @@ struct qcs *qcc_get_qcs(struct qcc *qcc, uint64_t id)
  */
 static int qcc_decode_qcs(struct qcc *qcc, struct qcs *qcs)
 {
+	ssize_t ret;
+	int fin = 0;
+
 	TRACE_ENTER(QMUX_EV_QCS_RECV, qcc->conn, qcs);
 
-	if (qcc->app_ops->decode_qcs(qcs, qcs->flags & QC_SF_FIN_RECV, qcc->ctx)) {
+	/* Signal FIN to application if STREAM FIN received and there is no gap
+	 * in the Rx buffer.
+	 */
+	if (qcs->flags & QC_SF_FIN_RECV && !ncb_is_fragmented(&qcs->rx.ncbuf))
+		fin = 1;
+
+	ret = qcc->app_ops->decode_qcs(qcs, fin);
+	if (ret < 0) {
 		TRACE_DEVEL("leaving on decoding error", QMUX_EV_QCS_RECV, qcc->conn, qcs);
 		return 1;
 	}
@@ -551,7 +561,10 @@ int qcc_recv(struct qcc *qcc, uint64_t id, uint64_t len, uint64_t offset,
 
 	TRACE_DEVEL("newly received offset", QMUX_EV_QCC_RECV|QMUX_EV_QCS_RECV, qcc->conn, qcs);
 	if (offset < qcs->rx.offset) {
-		len -= qcs->rx.offset - offset;
+		size_t diff = qcs->rx.offset - offset;
+
+		len -= diff;
+		data += diff;
 		offset = qcs->rx.offset;
 	}
 
@@ -850,11 +863,13 @@ static int qcs_build_stream_frm(struct qcs *qcs, struct buffer *out, char fin,
 	total = b_data(out) - head;
 	BUG_ON(total < 0);
 
-	if (!total) {
+	if (!total && !fin) {
+		/* No need to send anything if total is NULL and no FIN to signal. */
 		TRACE_LEAVE(QMUX_EV_QCS_SEND, qcc->conn, qcs);
 		return 0;
 	}
-	BUG_ON(qcs->tx.sent_offset >= qcs->tx.offset);
+	BUG_ON((!total && qcs->tx.sent_offset > qcs->tx.offset) ||
+	       (total && qcs->tx.sent_offset >= qcs->tx.offset));
 	BUG_ON(qcs->tx.sent_offset + total > qcs->tx.offset);
 
 	frm = pool_zalloc(pool_head_quic_frame);
@@ -899,6 +914,16 @@ static int qcs_build_stream_frm(struct qcs *qcs, struct buffer *out, char fin,
 	return -1;
 }
 
+/* Check after transfering data from qcs.tx.buf if FIN must be set on the next
+ * STREAM frame for <qcs>.
+ *
+ * Returns true if FIN must be set else false.
+ */
+static int qcs_stream_fin(struct qcs *qcs)
+{
+	return qcs->flags & QC_SF_FIN_STREAM && !b_data(&qcs->tx.buf);
+}
+
 /* This function must be called by the upper layer to inform about the sending
  * of a STREAM frame for <qcs> instance. The frame is of <data> length and on
  * <offset>.
@@ -914,29 +939,36 @@ void qcc_streams_sent_done(struct qcs *qcs, uint64_t data, uint64_t offset)
 	/* check if the STREAM frame has already been notified. It can happen
 	 * for retransmission.
 	 */
-	if (offset + data <= qcs->tx.sent_offset)
+	if (offset + data < qcs->tx.sent_offset)
 		return;
 
 	diff = offset + data - qcs->tx.sent_offset;
+	if (diff) {
+		/* increase offset sum on connection */
+		qcc->tx.sent_offsets += diff;
+		BUG_ON_HOT(qcc->tx.sent_offsets > qcc->rfctl.md);
+		if (qcc->tx.sent_offsets == qcc->rfctl.md)
+			qcc->flags |= QC_CF_BLK_MFCTL;
 
-	/* increase offset sum on connection */
-	qcc->tx.sent_offsets += diff;
-	BUG_ON_HOT(qcc->tx.sent_offsets > qcc->rfctl.md);
-	if (qcc->tx.sent_offsets == qcc->rfctl.md)
-		qcc->flags |= QC_CF_BLK_MFCTL;
+		/* increase offset on stream */
+		qcs->tx.sent_offset += diff;
+		BUG_ON_HOT(qcs->tx.sent_offset > qcs->tx.msd);
+		BUG_ON_HOT(qcs->tx.sent_offset > qcs->tx.offset);
+		if (qcs->tx.sent_offset == qcs->tx.msd)
+			qcs->flags |= QC_SF_BLK_SFCTL;
 
-	/* increase offset on stream */
-	qcs->tx.sent_offset += diff;
-	BUG_ON_HOT(qcs->tx.sent_offset > qcs->tx.msd);
-	BUG_ON_HOT(qcs->tx.sent_offset > qcs->tx.offset);
-	if (qcs->tx.sent_offset == qcs->tx.msd)
-		qcs->flags |= QC_SF_BLK_SFCTL;
+		if (qcs->tx.offset == qcs->tx.sent_offset &&
+		    b_full(&qcs->stream->buf->buf)) {
+			qc_stream_buf_release(qcs->stream);
+			/* prepare qcs for immediate send retry if data to send */
+			if (b_data(&qcs->tx.buf))
+				LIST_APPEND(&qcc->send_retry_list, &qcs->el);
+		}
+	}
 
-	if (qcs->tx.offset == qcs->tx.sent_offset && b_full(&qcs->stream->buf->buf)) {
-		qc_stream_buf_release(qcs->stream);
-		/* prepare qcs for immediate send retry if data to send */
-		if (b_data(&qcs->tx.buf))
-			LIST_APPEND(&qcc->send_retry_list, &qcs->el);
+	if (qcs->tx.offset == qcs->tx.sent_offset && qcs_stream_fin(qcs)) {
+		/* Reset flag to not emit multiple FIN STREAM frames. */
+		qcs->flags &= ~QC_SF_FIN_STREAM;
 	}
 }
 
@@ -1029,6 +1061,7 @@ static int _qc_send_qcs(struct qcs *qcs, struct list *frms)
 	struct buffer *buf = &qcs->tx.buf;
 	struct buffer *out = qc_stream_buf_get(qcs->stream);
 	int xfer = 0;
+	char fin = 0;
 
 	/* Allocate <out> buffer if necessary. */
 	if (!out) {
@@ -1057,14 +1090,12 @@ static int _qc_send_qcs(struct qcs *qcs, struct list *frms)
 	/* out buffer cannot be emptied if qcs offsets differ. */
 	BUG_ON(!b_data(out) && qcs->tx.sent_offset != qcs->tx.offset);
 
+	/* FIN is set if all incoming data were transfered. */
+	fin = qcs_stream_fin(qcs);
+
 	/* Build a new STREAM frame with <out> buffer. */
-	if (qcs->tx.sent_offset != qcs->tx.offset) {
+	if (qcs->tx.sent_offset != qcs->tx.offset || fin) {
 		int ret;
-		char fin = !!(qcs->flags & QC_SF_FIN_STREAM);
-
-		/* FIN is set if all incoming data were transfered. */
-		fin = !!(fin && !b_data(buf));
-
 		ret = qcs_build_stream_frm(qcs, out, fin, frms);
 		if (ret < 0) { ABORT_NOW(); /* TODO handle this properly */ }
 	}
@@ -1654,13 +1685,13 @@ static void qmux_trace_frm(const struct quic_frame *frm)
 {
 	switch (frm->type) {
 	case QUIC_FT_MAX_STREAMS_BIDI:
-		chunk_appendf(&trace_buf, " max_streams=%lu",
-		              frm->max_streams_bidi.max_streams);
+		chunk_appendf(&trace_buf, " max_streams=%llu",
+		              (ull)frm->max_streams_bidi.max_streams);
 		break;
 
 	case QUIC_FT_MAX_STREAMS_UNI:
-		chunk_appendf(&trace_buf, " max_streams=%lu",
-		              frm->max_streams_uni.max_streams);
+		chunk_appendf(&trace_buf, " max_streams=%llu",
+		              (ull)frm->max_streams_uni.max_streams);
 		break;
 
 	default:
@@ -1685,11 +1716,11 @@ static void qmux_trace(enum trace_level level, uint64_t mask,
 		chunk_appendf(&trace_buf, " : qcc=%p(F)", qcc);
 
 		if (qcs)
-			chunk_appendf(&trace_buf, " qcs=%p(%lu)", qcs, qcs->id);
+			chunk_appendf(&trace_buf, " qcs=%p(%llu)", qcs, (ull)qcs->id);
 
 		if (mask & QMUX_EV_QCC_NQCS) {
 			const uint64_t *id = a3;
-			chunk_appendf(&trace_buf, " id=%lu", *id);
+			chunk_appendf(&trace_buf, " id=%llu", (ull)*id);
 		}
 
 		if (mask & QMUX_EV_SEND_FRM)
@@ -1697,14 +1728,14 @@ static void qmux_trace(enum trace_level level, uint64_t mask,
 
 		if (mask & QMUX_EV_QCS_XFER_DATA) {
 			const struct qcs_xfer_data_trace_arg *arg = a3;
-			chunk_appendf(&trace_buf, " prep=%lu xfer=%d",
-			              arg->prep, arg->xfer);
+			chunk_appendf(&trace_buf, " prep=%llu xfer=%d",
+			              (ull)arg->prep, arg->xfer);
 		}
 
 		if (mask & QMUX_EV_QCS_BUILD_STRM) {
 			const struct qcs_build_stream_trace_arg *arg = a3;
-			chunk_appendf(&trace_buf, " len=%lu fin=%d offset=%lu",
-			              arg->len, arg->fin, arg->offset);
+			chunk_appendf(&trace_buf, " len=%llu fin=%d offset=%llu",
+			              (ull)arg->len, arg->fin, (ull)arg->offset);
 		}
 	}
 }
