@@ -1,7 +1,7 @@
 /*
  * AF_INET/AF_INET6 QUIC protocol layer.
  *
- * Copyright 2020 Frédéric Lécaille <flecaille@haproxy.com>
+ * Copyright 2020 Frederic Lecaille <flecaille@haproxy.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,7 +12,6 @@
 
 #include <ctype.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -45,8 +44,15 @@
 #include <haproxy/quic_sock.h>
 #include <haproxy/sock_inet.h>
 #include <haproxy/tools.h>
-#include <haproxy/xprt_quic-t.h>
+#include <haproxy/xprt_quic.h>
 
+/* per-thread quic datagram handlers */
+struct quic_dghdlr *quic_dghdlrs;
+
+/* Size of the internal buffer of QUIC RX buffer at the fd level */
+#define QUIC_RX_BUFSZ  (1UL << 18)
+
+DECLARE_STATIC_POOL(pool_head_quic_rxbuf, "quic_rxbuf_pool", QUIC_RX_BUFSZ);
 
 static void quic_add_listener(struct protocol *proto, struct listener *listener);
 static int quic_bind_listener(struct listener *listener, char *errmsg, int errlen);
@@ -59,7 +65,7 @@ struct protocol proto_quic4 = {
 	.name           = "quic4",
 
 	/* connection layer */
-	.ctrl_type      = SOCK_STREAM,
+	.xprt_type      = PROTO_TYPE_STREAM,
 	.listen         = quic_bind_listener,
 	.enable         = quic_enable_listener,
 	.disable        = quic_disable_listener,
@@ -68,6 +74,8 @@ struct protocol proto_quic4 = {
 	.suspend        = default_suspend_listener,
 	.resume         = default_resume_listener,
 	.accept_conn    = quic_sock_accept_conn,
+	.get_src        = quic_sock_get_src,
+	.get_dst        = quic_sock_get_dst,
 	.connect        = quic_connect_server,
 
 	/* binding layer */
@@ -97,7 +105,7 @@ struct protocol proto_quic6 = {
 	.name            = "quic6",
 
 	/* connection layer */
-	.ctrl_type      = SOCK_STREAM,
+	.xprt_type      = PROTO_TYPE_STREAM,
 	.listen         = quic_bind_listener,
 	.enable         = quic_enable_listener,
 	.disable        = quic_disable_listener,
@@ -106,6 +114,8 @@ struct protocol proto_quic6 = {
 	.suspend        = default_suspend_listener,
 	.resume         = default_resume_listener,
 	.accept_conn    = quic_sock_accept_conn,
+	.get_src        = quic_sock_get_src,
+	.get_dst        = quic_sock_get_dst,
 	.connect        = quic_connect_server,
 
 	/* binding layer */
@@ -257,15 +267,17 @@ int quic_connect_server(struct connection *conn, int flags)
 	struct conn_src *src;
 	struct sockaddr_storage *addr;
 
+	BUG_ON(!conn->dst);
+
 	conn->flags |= CO_FL_WAIT_L4_CONN; /* connection in progress */
 
 	switch (obj_type(conn->target)) {
 	case OBJ_TYPE_PROXY:
-		be = objt_proxy(conn->target);
+		be = __objt_proxy(conn->target);
 		srv = NULL;
 		break;
 	case OBJ_TYPE_SERVER:
-		srv = objt_server(conn->target);
+		srv = __objt_server(conn->target);
 		be = srv->proxy;
 		break;
 	default:
@@ -273,11 +285,6 @@ int quic_connect_server(struct connection *conn, int flags)
 		return SF_ERR_INTERNAL;
 	}
 
-	if (!conn->dst) {
-		conn->flags |= CO_FL_ERROR;
-		return SF_ERR_INTERNAL;
-	}
-
 	fd = conn->handle.fd = sock_create_server_socket(conn);
 
 	if (fd == -1) {
@@ -323,7 +330,7 @@ int quic_connect_server(struct connection *conn, int flags)
 		return SF_ERR_PRXCOND; /* it is a configuration limit */
 	}
 
-	if ((fcntl(fd, F_SETFL, O_NONBLOCK)==-1)) {
+	if (fd_set_nonblock(fd) == -1) {
 		qfprintf(stderr,"Cannot set client socket to non blocking mode.\n");
 		close(fd);
 		conn->err_code = CO_ER_SOCK_ERR;
@@ -331,7 +338,7 @@ int quic_connect_server(struct connection *conn, int flags)
 		return SF_ERR_INTERNAL;
 	}
 
-	if (master == 1 && (fcntl(fd, F_SETFD, FD_CLOEXEC) == -1)) {
+	if (master == 1 && fd_set_cloexec(fd) == -1) {
 		ha_alert("Cannot set CLOEXEC on client socket.\n");
 		close(fd);
 		conn->err_code = CO_ER_SOCK_ERR;
@@ -456,9 +463,9 @@ int quic_connect_server(struct connection *conn, int flags)
 			/* should normally not happen but if so, indicates that it's OK */
 			conn->flags &= ~CO_FL_WAIT_L4_CONN;
 		}
-		else if (errno == EAGAIN || errno == EADDRINUSE || errno == EADDRNOTAVAIL) {
+		else if (errno == EAGAIN || errno == EWOULDBLOCK || errno == EADDRINUSE || errno == EADDRNOTAVAIL) {
 			char *msg;
-			if (errno == EAGAIN || errno == EADDRNOTAVAIL) {
+			if (errno == EAGAIN || errno == EWOULDBLOCK || errno == EADDRNOTAVAIL) {
 				msg = "no free ports";
 				conn->err_code = CO_ER_FREE_PORTS;
 			}
@@ -498,8 +505,6 @@ int quic_connect_server(struct connection *conn, int flags)
 		conn->flags &= ~CO_FL_WAIT_L4_CONN;
 	}
 
-	conn->flags |= CO_FL_ADDR_TO_SET;
-
 	conn_ctrl_init(conn);       /* registers the FD */
 	HA_ATOMIC_OR(&fdtab[fd].state, FD_LINGER_RISK);  /* close hard if needed */
 
@@ -518,10 +523,9 @@ int quic_connect_server(struct connection *conn, int flags)
  */
 static void quic_add_listener(struct protocol *proto, struct listener *listener)
 {
-	MT_LIST_INIT(&listener->rx.pkts);
-	listener->rx.odcids = EB_ROOT_UNIQUE;
-	listener->rx.cids = EB_ROOT_UNIQUE;
-	HA_RWLOCK_INIT(&listener->rx.cids_lock);
+	listener->flags |= LI_F_QUIC_LISTENER;
+	listener->rx.flags |= RX_F_LOCAL_ACCEPT;
+
 	default_add_listener(proto, listener);
 }
 
@@ -540,18 +544,26 @@ static int quic_alloc_tx_rings_listener(struct listener *l)
 	MT_LIST_INIT(&l->rx.tx_qring_list);
 	for (i = 0; i < global.nbthread; i++) {
 		unsigned char *buf;
-		struct qring *qr = &l->rx.tx_qrings[i];
+		struct qring *qr;
+
+		qr = calloc(1, sizeof *qr);
+		if (!qr)
+			goto err;
 
 		buf = pool_alloc(pool_head_quic_tx_ring);
-		if (!buf)
+		if (!buf) {
+			free(qr);
 			goto err;
+		}
 
 		qr->cbuf = cbuf_new(buf, QUIC_TX_RING_BUFSZ);
 		if (!qr->cbuf) {
 			pool_free(pool_head_quic_tx_ring, buf);
+			free(qr);
 			goto err;
 		}
 
+        l->rx.tx_qrings[i] = qr;
 		MT_LIST_APPEND(&l->rx.tx_qring_list, &qr->mt_list);
 	}
 
@@ -561,6 +573,7 @@ static int quic_alloc_tx_rings_listener(struct listener *l)
 	while ((qr = MT_LIST_POP(&l->rx.tx_qring_list, typeof(qr), mt_list))) {
 		pool_free(pool_head_quic_tx_ring, qr->cbuf->buf);
 		cbuf_free(qr->cbuf);
+		free(qr);
 	}
 	free(l->rx.tx_qrings);
 	return 0;
@@ -581,21 +594,32 @@ static int quic_alloc_rxbufs_listener(struct listener *l)
 	MT_LIST_INIT(&l->rx.rxbuf_list);
 	for (i = 0; i < global.nbthread; i++) {
 		char *buf;
+		struct rxbuf *rxbuf;
 
-		rxbuf = &l->rx.rxbufs[i];
-		buf = pool_alloc(pool_head_quic_rxbuf);
-		if (!buf)
+		rxbuf = calloc(1, sizeof *rxbuf);
+		if (!rxbuf)
 			goto err;
 
+		buf = pool_alloc(pool_head_quic_rxbuf);
+		if (!buf) {
+			free(rxbuf);
+			goto err;
+		}
+
+		l->rx.rxbufs[i] = rxbuf;
+
 		rxbuf->buf = b_make(buf, QUIC_RX_BUFSZ, 0, 0);
+		LIST_INIT(&rxbuf->dgrams);
 		MT_LIST_APPEND(&l->rx.rxbuf_list, &rxbuf->mt_list);
 	}
 
 	return 1;
 
  err:
-	while ((rxbuf = MT_LIST_POP(&l->rx.rxbuf_list, typeof(rxbuf), mt_list)))
+	while ((rxbuf = MT_LIST_POP(&l->rx.rxbuf_list, typeof(rxbuf), mt_list))) {
 		pool_free(pool_head_quic_rxbuf, rxbuf->buf.area);
+		free(rxbuf);
+	}
 	free(l->rx.rxbufs);
 	return 0;
 }
@@ -675,6 +699,53 @@ static void quic_disable_listener(struct listener *l)
 		fd_stop_recv(l->rx.fd);
 }
 
+static int quic_alloc_dghdlrs(void)
+{
+	int i;
+
+	quic_dghdlrs = calloc(global.nbthread, sizeof(struct quic_dghdlr));
+	if (!quic_dghdlrs) {
+		ha_alert("Failed to allocate the quic datagram handlers.\n");
+		return 0;
+	}
+
+	for (i = 0; i < global.nbthread; i++) {
+		struct quic_dghdlr *dghdlr = &quic_dghdlrs[i];
+
+		dghdlr->task = tasklet_new();
+		if (!dghdlr->task) {
+			ha_alert("Failed to allocate the quic datagram handler on thread %d.\n", i);
+			return 0;
+		}
+
+		tasklet_set_tid(dghdlr->task, i);
+		dghdlr->task->context = dghdlr;
+		dghdlr->task->process = quic_lstnr_dghdlr;
+
+		dghdlr->odcids = EB_ROOT_UNIQUE;
+		dghdlr->cids = EB_ROOT_UNIQUE;
+
+		MT_LIST_INIT(&dghdlr->dgrams);
+	}
+
+	return 1;
+}
+REGISTER_POST_CHECK(quic_alloc_dghdlrs);
+
+static int quic_deallocate_dghdlrs(void)
+{
+	int i;
+
+	if (quic_dghdlrs) {
+		for (i = 0; i < global.nbthread; ++i)
+			tasklet_free(quic_dghdlrs[i].task);
+		free(quic_dghdlrs);
+	}
+
+	return 1;
+}
+REGISTER_POST_DEINIT(quic_deallocate_dghdlrs);
+
 /*
  * Local variables:
  *  c-indent-level: 8
