@@ -24,7 +24,7 @@
 
 /* List head of all registered protocols */
 static struct list protocols = LIST_HEAD_INIT(protocols);
-struct protocol *__protocol_by_family[AF_CUST_MAX][2][2] __read_mostly = { };
+struct protocol *__protocol_by_family[AF_CUST_MAX][PROTO_NUM_TYPES][2] __read_mostly = { };
 
 /* This is the global spinlock we may need to register/unregister listeners or
  * protocols. Its main purpose is in fact to serialize the rare stop/deinit()
@@ -35,12 +35,16 @@ __decl_spinlock(proto_lock);
 /* Registers the protocol <proto> */
 void protocol_register(struct protocol *proto)
 {
+	int sock_domain = proto->fam->sock_domain;
+
+	BUG_ON(sock_domain < 0 || sock_domain >= AF_CUST_MAX);
+	BUG_ON(proto->proto_type >= PROTO_NUM_TYPES);
+
 	HA_SPIN_LOCK(PROTO_LOCK, &proto_lock);
 	LIST_APPEND(&protocols, &proto->list);
-	if (proto->fam->sock_domain >= 0 && proto->fam->sock_domain < AF_CUST_MAX)
-		__protocol_by_family[proto->fam->sock_domain]
-			[proto->sock_type == SOCK_DGRAM]
-			[proto->ctrl_type == SOCK_DGRAM] = proto;
+	__protocol_by_family[sock_domain]
+	                    [proto->proto_type]
+	                    [proto->xprt_type == PROTO_TYPE_DGRAM] = proto;
 	HA_SPIN_UNLOCK(PROTO_LOCK, &proto_lock);
 }
 
@@ -63,7 +67,7 @@ int protocol_bind_all(int verbose)
 	struct protocol *proto;
 	struct listener *listener;
 	struct receiver *receiver;
-	char msg[100];
+	char msg[1000];
 	char *errmsg;
 	int err, lerr;
 
@@ -81,15 +85,15 @@ int protocol_bind_all(int verbose)
 				struct proxy *px = listener->bind_conf->frontend;
 
 				if (lerr & ERR_ALERT)
-					ha_alert("Starting %s %s: %s\n",
+					ha_alert("Binding [%s:%d] for %s %s: %s\n",
+					         listener->bind_conf->file, listener->bind_conf->line,
 						 proxy_type_str(px), px->id, errmsg);
 				else if (lerr & ERR_WARN)
-					ha_warning("Starting %s %s: %s\n",
+					ha_warning("Binding [%s:%d] for %s %s: %s\n",
+					           listener->bind_conf->file, listener->bind_conf->line,
 						   proxy_type_str(px), px->id, errmsg);
-			}
-			if (lerr != ERR_NONE)
 				ha_free(&errmsg);
-
+			}
 			if (lerr & ERR_ABORT)
 				break;
 
@@ -105,10 +109,12 @@ int protocol_bind_all(int verbose)
 				struct proxy *px = listener->bind_conf->frontend;
 
 				if (lerr & ERR_ALERT)
-					ha_alert("Starting %s %s: %s\n",
+					ha_alert("Starting [%s:%d] for %s %s: %s\n",
+					         listener->bind_conf->file, listener->bind_conf->line,
 						 proxy_type_str(px), px->id, msg);
 				else if (lerr & ERR_WARN)
-					ha_warning("Starting %s %s: %s\n",
+					ha_warning("Starting [%s:%d] for %s %s: %s\n",
+					           listener->bind_conf->file, listener->bind_conf->line,
 						   proxy_type_str(px), px->id, msg);
 			}
 			if (lerr & ERR_ABORT)
@@ -142,11 +148,9 @@ int protocol_unbind_all(void)
 	return err;
 }
 
-/* stops all listeners of all registered protocols, except when the belong to a
- * proxy configured with a grace time. This will normally catch every single
- * listener, all protocols included, and the grace ones will have to be handled
- * by the proxy stopping loop. This is to be used during soft_stop() only. It
- * does not return any error.
+/* stops all listeners of all registered protocols. This will normally catch
+ * every single listener, all protocols included. This is to be used during
+ * soft_stop() only. It does not return any error.
  */
 void protocol_stop_now(void)
 {
@@ -156,16 +160,15 @@ void protocol_stop_now(void)
 	HA_SPIN_LOCK(PROTO_LOCK, &proto_lock);
 	list_for_each_entry(proto, &protocols, list) {
 		list_for_each_entry_safe(listener, lback, &proto->receivers, rx.proto_list)
-			if (!listener->bind_conf->frontend->grace)
-				stop_listener(listener, 0, 1, 0);
+			stop_listener(listener, 0, 1, 0);
 	}
 	HA_SPIN_UNLOCK(PROTO_LOCK, &proto_lock);
 }
 
-/* suspends all listeners of all registered protocols. This is typically
+/* pauses all listeners of all registered protocols. This is typically
  * used on SIG_TTOU to release all listening sockets for the time needed to
- * try to bind a new process. The listeners enter LI_PAUSED or LI_ASSIGNED.
- * It returns ERR_NONE, with ERR_FATAL on failure.
+ * try to bind a new process. The listeners enter LI_PAUSED. It returns
+ * ERR_NONE, with ERR_FATAL on failure.
  */
 int protocol_pause_all(void)
 {
@@ -177,7 +180,7 @@ int protocol_pause_all(void)
 	HA_SPIN_LOCK(PROTO_LOCK, &proto_lock);
 	list_for_each_entry(proto, &protocols, list) {
 		list_for_each_entry(listener, &proto->receivers, rx.proto_list)
-			if (!suspend_listener(listener, 0, 0))
+			if (!pause_listener(listener))
 				err |= ERR_FATAL;
 	}
 	HA_SPIN_UNLOCK(PROTO_LOCK, &proto_lock);
@@ -199,7 +202,7 @@ int protocol_resume_all(void)
 	HA_SPIN_LOCK(PROTO_LOCK, &proto_lock);
 	list_for_each_entry(proto, &protocols, list) {
 		list_for_each_entry(listener, &proto->receivers, rx.proto_list)
-			if (!resume_listener(listener, 0, 0))
+			if (!resume_listener(listener))
 				err |= ERR_FATAL;
 	}
 	HA_SPIN_UNLOCK(PROTO_LOCK, &proto_lock);
