@@ -20,6 +20,7 @@
 
 #include <import/ist.h>
 #include <haproxy/api.h>
+#include <haproxy/applet.h>
 #include <haproxy/cfgparse.h>
 #include <haproxy/cli.h>
 #include <haproxy/errors.h>
@@ -27,9 +28,10 @@
 #include <haproxy/log.h>
 #include <haproxy/proxy.h>
 #include <haproxy/ring.h>
+#include <haproxy/sc_strm.h>
 #include <haproxy/signal.h>
 #include <haproxy/sink.h>
-#include <haproxy/stream_interface.h>
+#include <haproxy/stconn.h>
 #include <haproxy/time.h>
 #include <haproxy/tools.h>
 
@@ -205,7 +207,7 @@ int sink_announce_dropped(struct sink *sink, int facility)
 
 		if (!metadata[LOG_META_HOST].len) {
 			if (global.log_send_hostname)
-				metadata[LOG_META_HOST] = ist2(global.log_send_hostname, strlen(global.log_send_hostname));
+				metadata[LOG_META_HOST] = ist(global.log_send_hostname);
 		}
 
 		if (!metadata[LOG_META_TAG].len)
@@ -232,6 +234,7 @@ int sink_announce_dropped(struct sink *sink, int facility)
 static int cli_parse_show_events(char **args, char *payload, struct appctx *appctx, void *private)
 {
 	struct sink *sink;
+	uint ring_flags;
 	int arg;
 
 	args++; // make args[1] the 1st arg
@@ -262,17 +265,18 @@ static int cli_parse_show_events(char **args, char *payload, struct appctx *appc
 	if (sink->type != SINK_TYPE_BUFFER)
 		return cli_msg(appctx, LOG_NOTICE, "Nothing to report for this sink");
 
+	ring_flags = 0;
 	for (arg = 2; *args[arg]; arg++) {
 		if (strcmp(args[arg], "-w") == 0)
-			appctx->ctx.cli.i0 |= 1; // wait mode
+			ring_flags |= RING_WF_WAIT_MODE;
 		else if (strcmp(args[arg], "-n") == 0)
-			appctx->ctx.cli.i0 |= 2; // seek to new
+			ring_flags |= RING_WF_SEEK_NEW;
 		else if (strcmp(args[arg], "-nw") == 0 || strcmp(args[arg], "-wn") == 0)
-			appctx->ctx.cli.i0 |= 3; // seek to new + wait
+			ring_flags |= RING_WF_WAIT_MODE | RING_WF_SEEK_NEW;
 		else
 			return cli_err(appctx, "unknown option");
 	}
-	return ring_attach_cli(sink->ctx.ring, appctx);
+	return ring_attach_cli(sink->ctx.ring, appctx, ring_flags);
 }
 
 /* Pre-configures a ring proxy to emit connections */
@@ -290,14 +294,15 @@ void sink_setup_proxy(struct proxy *px)
 }
 
 /*
- * IO Handler to handle message push to syslog tcp server
+ * IO Handler to handle message push to syslog tcp server.
+ * It takes its context from appctx->svcctx.
  */
 static void sink_forward_io_handler(struct appctx *appctx)
 {
-	struct stream_interface *si = appctx->owner;
-	struct stream *s = si_strm(si);
+	struct stconn *sc = appctx_sc(appctx);
+	struct stream *s = __sc_strm(sc);
 	struct sink *sink = strm_fe(s)->parent;
-	struct sink_forward_target *sft = appctx->ctx.sft.ptr;
+	struct sink_forward_target *sft = appctx->svcctx;
 	struct ring *ring = sink->ctx.ring;
 	struct buffer *buf = &ring->buf;
 	uint64_t msg_len;
@@ -312,25 +317,25 @@ static void sink_forward_io_handler(struct appctx *appctx)
 	 * and we don't want expire on this case
 	 * with a syslog server
 	 */
-	si_oc(si)->rex = TICK_ETERNITY;
+	sc_oc(sc)->rex = TICK_ETERNITY;
 	/* rto should not change but it seems the case */
-	si_oc(si)->rto = TICK_ETERNITY;
+	sc_oc(sc)->rto = TICK_ETERNITY;
 
 	/* an error was detected */
-	if (unlikely(si_ic(si)->flags & (CF_WRITE_ERROR|CF_SHUTW)))
+	if (unlikely(sc_ic(sc)->flags & (CF_WRITE_ERROR|CF_SHUTW)))
 		goto close;
 
 	/* con closed by server side */
-	if ((si_oc(si)->flags & CF_SHUTW))
+	if ((sc_oc(sc)->flags & CF_SHUTW))
 		goto close;
 
 	/* if the connection is not established, inform the stream that we want
 	 * to be notified whenever the connection completes.
 	 */
-	if (si_opposite(si)->state < SI_ST_EST) {
-		si_cant_get(si);
-		si_rx_conn_blk(si);
-		si_rx_endp_more(si);
+	if (sc_opposite(sc)->state < SC_ST_EST) {
+		applet_need_more_data(appctx);
+		se_need_remote_conn(appctx->sedesc);
+		applet_have_more_data(appctx);
 		return;
 	}
 
@@ -366,7 +371,7 @@ static void sink_forward_io_handler(struct appctx *appctx)
 	 * the message so that we can take our reference there if we have to
 	 * stop before the end (ret=0).
 	 */
-	if (si_opposite(si)->state == SI_ST_EST) {
+	if (sc_opposite(sc)->state == SC_ST_EST) {
 		/* we were already there, adjust the offset to be relative to
 		 * the buffer's head and remove us from the counter.
 		 */
@@ -394,8 +399,7 @@ static void sink_forward_io_handler(struct appctx *appctx)
 			trash.data += len;
 			trash.area[trash.data++] = '\n';
 
-			if (ci_putchk(si_ic(si), &trash) == -1) {
-				si_rx_room_blk(si);
+			if (applet_putchk(appctx, &trash) == -1) {
 				ret = 0;
 				break;
 			}
@@ -413,30 +417,31 @@ static void sink_forward_io_handler(struct appctx *appctx)
 		HA_RWLOCK_WRLOCK(LOGSRV_LOCK, &ring->lock);
 		LIST_APPEND(&ring->waiters, &appctx->wait_entry);
 		HA_RWLOCK_WRUNLOCK(LOGSRV_LOCK, &ring->lock);
-		si_rx_endp_done(si);
+		applet_have_no_more_data(appctx);
 	}
 	HA_SPIN_UNLOCK(SFT_LOCK, &sft->lock);
 
 	/* always drain data from server */
-	co_skip(si_oc(si), si_oc(si)->output);
+	co_skip(sc_oc(sc), sc_oc(sc)->output);
 	return;
 
 close:
-	si_shutw(si);
-	si_shutr(si);
-	si_ic(si)->flags |= CF_READ_NULL;
+	sc_shutw(sc);
+	sc_shutr(sc);
+	sc_ic(sc)->flags |= CF_READ_NULL;
 }
 
 /*
  * IO Handler to handle message push to syslog tcp server
  * using octet counting frames
+ * It takes its context from appctx->svcctx.
  */
 static void sink_forward_oc_io_handler(struct appctx *appctx)
 {
-	struct stream_interface *si = appctx->owner;
-	struct stream *s = si_strm(si);
+	struct stconn *sc = appctx_sc(appctx);
+	struct stream *s = __sc_strm(sc);
 	struct sink *sink = strm_fe(s)->parent;
-	struct sink_forward_target *sft = appctx->ctx.sft.ptr;
+	struct sink_forward_target *sft = appctx->svcctx;
 	struct ring *ring = sink->ctx.ring;
 	struct buffer *buf = &ring->buf;
 	uint64_t msg_len;
@@ -452,25 +457,25 @@ static void sink_forward_oc_io_handler(struct appctx *appctx)
 	 * and we don't want expire on this case
 	 * with a syslog server
 	 */
-	si_oc(si)->rex = TICK_ETERNITY;
+	sc_oc(sc)->rex = TICK_ETERNITY;
 	/* rto should not change but it seems the case */
-	si_oc(si)->rto = TICK_ETERNITY;
+	sc_oc(sc)->rto = TICK_ETERNITY;
 
 	/* an error was detected */
-	if (unlikely(si_ic(si)->flags & (CF_WRITE_ERROR|CF_SHUTW)))
+	if (unlikely(sc_ic(sc)->flags & (CF_WRITE_ERROR|CF_SHUTW)))
 		goto close;
 
 	/* con closed by server side */
-	if ((si_oc(si)->flags & CF_SHUTW))
+	if ((sc_oc(sc)->flags & CF_SHUTW))
 		goto close;
 
 	/* if the connection is not established, inform the stream that we want
 	 * to be notified whenever the connection completes.
 	 */
-	if (si_opposite(si)->state < SI_ST_EST) {
-		si_cant_get(si);
-		si_rx_conn_blk(si);
-		si_rx_endp_more(si);
+	if (sc_opposite(sc)->state < SC_ST_EST) {
+		applet_need_more_data(appctx);
+		se_need_remote_conn(appctx->sedesc);
+		applet_have_more_data(appctx);
 		return;
 	}
 
@@ -506,7 +511,7 @@ static void sink_forward_oc_io_handler(struct appctx *appctx)
 	 * the message so that we can take our reference there if we have to
 	 * stop before the end (ret=0).
 	 */
-	if (si_opposite(si)->state == SI_ST_EST) {
+	if (sc_opposite(sc)->state == SC_ST_EST) {
 		/* we were already there, adjust the offset to be relative to
 		 * the buffer's head and remove us from the counter.
 		 */
@@ -538,8 +543,7 @@ static void sink_forward_oc_io_handler(struct appctx *appctx)
 
 			trash.data += b_getblk(buf, p + 1, msg_len, ofs + cnt);
 
-			if (ci_putchk(si_ic(si), &trash) == -1) {
-				si_rx_room_blk(si);
+			if (applet_putchk(appctx, &trash) == -1) {
 				ret = 0;
 				break;
 			}
@@ -557,37 +561,25 @@ static void sink_forward_oc_io_handler(struct appctx *appctx)
 		HA_RWLOCK_WRLOCK(LOGSRV_LOCK, &ring->lock);
 		LIST_APPEND(&ring->waiters, &appctx->wait_entry);
 		HA_RWLOCK_WRUNLOCK(LOGSRV_LOCK, &ring->lock);
-		si_rx_endp_done(si);
+		applet_have_no_more_data(appctx);
 	}
 	HA_SPIN_UNLOCK(SFT_LOCK, &sft->lock);
 
 	/* always drain data from server */
-	co_skip(si_oc(si), si_oc(si)->output);
+	co_skip(sc_oc(sc), sc_oc(sc)->output);
 	return;
 
 close:
-	si_shutw(si);
-	si_shutr(si);
-	si_ic(si)->flags |= CF_READ_NULL;
+	sc_shutw(sc);
+	sc_shutr(sc);
+	sc_ic(sc)->flags |= CF_READ_NULL;
 }
 
 void __sink_forward_session_deinit(struct sink_forward_target *sft)
 {
-	struct stream_interface *si;
-	struct stream *s;
+	struct stream *s = appctx_strm(sft->appctx);
 	struct sink *sink;
 
-	if (!sft->appctx)
-		return;
-
-	si = sft->appctx->owner;
-	if (!si)
-		return;
-
-	s = si_strm(si);
-	if (!s)
-		return;
-
 	sink = strm_fe(s)->parent;
 	if (!sink)
 		return;
@@ -600,10 +592,47 @@ void __sink_forward_session_deinit(struct sink_forward_target *sft)
 	task_wakeup(sink->forward_task, TASK_WOKEN_MSG);
 }
 
+static int sink_forward_session_init(struct appctx *appctx)
+{
+	struct sink_forward_target *sft = appctx->svcctx;
+	struct stream *s;
+	struct sockaddr_storage *addr = NULL;
+
+	if (!sockaddr_alloc(&addr, &sft->srv->addr, sizeof(sft->srv->addr)))
+		goto out_error;
+
+	if (appctx_finalize_startup(appctx, sft->sink->forward_px, &BUF_NULL) == -1)
+		goto out_free_addr;
+
+	s = appctx_strm(appctx);
+	s->scb->dst = addr;
+	s->scb->flags |= SC_FL_NOLINGER;
+
+	s->target = &sft->srv->obj_type;
+	s->flags = SF_ASSIGNED;
+
+	s->do_log = NULL;
+	s->uniq_id = 0;
+
+	s->res.flags |= CF_READ_DONTWAIT;
+	/* for rto and rex to eternity to not expire on idle recv:
+	 * We are using a syslog server.
+	 */
+	s->res.rto = TICK_ETERNITY;
+	s->res.rex = TICK_ETERNITY;
+	sft->appctx = appctx;
+
+	return 0;
+
+ out_free_addr:
+	sockaddr_free(&addr);
+ out_error:
+	return -1;
+}
 
 static void sink_forward_session_release(struct appctx *appctx)
 {
-	struct sink_forward_target *sft = appctx->ctx.peers.ptr;
+	struct sink_forward_target *sft = appctx->svcctx;
 
 	if (!sft)
 		return;
@@ -618,6 +647,7 @@ static struct applet sink_forward_applet = {
 	.obj_type = OBJ_TYPE_APPLET,
 	.name = "<SINKFWD>", /* used for logging */
 	.fct = sink_forward_io_handler,
+	.init = sink_forward_session_init,
 	.release = sink_forward_session_release,
 };
 
@@ -625,68 +655,35 @@ static struct applet sink_forward_oc_applet = {
 	.obj_type = OBJ_TYPE_APPLET,
 	.name = "<SINKFWDOC>", /* used for logging */
 	.fct = sink_forward_oc_io_handler,
+	.init = sink_forward_session_init,
 	.release = sink_forward_session_release,
 };
 
 /*
  * Create a new peer session in assigned state (connect will start automatically)
+ * It sets its context into appctx->svcctx.
  */
 static struct appctx *sink_forward_session_create(struct sink *sink, struct sink_forward_target *sft)
 {
-	struct proxy *p = sink->forward_px;
 	struct appctx *appctx;
-	struct session *sess;
-	struct stream *s;
 	struct applet *applet = &sink_forward_applet;
 
 	if (sft->srv->log_proto == SRV_LOG_PROTO_OCTET_COUNTING)
 		applet = &sink_forward_oc_applet;
 
-	appctx = appctx_new(applet);
+	appctx = appctx_new_here(applet, NULL);
 	if (!appctx)
 		goto out_close;
+	appctx->svcctx = (void *)sft;
 
-	appctx->ctx.sft.ptr = (void *)sft;
-
-	sess = session_new(p, NULL, &appctx->obj_type);
-	if (!sess) {
-		ha_alert("out of memory in peer_session_create().\n");
+	if (appctx_init(appctx) == -1)
 		goto out_free_appctx;
-	}
 
-	if ((s = stream_new(sess, &appctx->obj_type, &BUF_NULL)) == NULL) {
-		ha_alert("Failed to initialize stream in peer_session_create().\n");
-		goto out_free_sess;
-	}
-
-
-	s->target = &sft->srv->obj_type;
-	if (!sockaddr_alloc(&s->si[1].dst, &sft->srv->addr, sizeof(sft->srv->addr)))
-		goto out_free_strm;
-	s->flags = SF_ASSIGNED|SF_ADDR_SET;
-	s->si[1].flags |= SI_FL_NOLINGER;
-
-	s->do_log = NULL;
-	s->uniq_id = 0;
-
-	s->res.flags |= CF_READ_DONTWAIT;
-	/* for rto and rex to eternity to not expire on idle recv:
-	 * We are using a syslog server.
-	 */
-	s->res.rto = TICK_ETERNITY;
-	s->res.rex = TICK_ETERNITY;
-	sft->appctx = appctx;
-	task_wakeup(s->task, TASK_WOKEN_INIT);
 	return appctx;
 
 	/* Error unrolling */
- out_free_strm:
-	LIST_DELETE(&s->list);
-	pool_free(pool_head_stream, s);
- out_free_sess:
-	session_free(sess);
  out_free_appctx:
-	appctx_free(appctx);
+	appctx_free_on_early_error(appctx);
  out_close:
 	return NULL;
 }
@@ -1007,6 +1004,7 @@ struct sink *sink_new_from_logsrv(struct logsrv *logsrv)
 	/* insert into sink_forward_targets
 	 * list into sink
 	 */
+	sft->sink = sink;
 	sft->next = sink->sft;
 	sink->sft = sft;
 
